import xlwings as xw
import numpy as np


@xw.func
def SUTB(SUT, HC, VC, AC, AdC=None):

    SUT = np.array(SUT)
    HC = np.array(HC)
    VC = np.array(VC)
    AC = np.array(AC)

    dimSUT = np.shape(SUT)
    tnV = dimSUT[0] * dimSUT[1]

    if (AdC is None):
        AdC = np.zeros((dimSUT[0], dimSUT[1]+1))
    else:
        AdC = np.array(AdC)

    tnAdC = int(np.shape(AdC)[0]/dimSUT[0])

    ACM = np.zeros((tnV, tnAdC))
    ACMr = np.zeros((tnAdC, 1))

    for i in range(0, tnAdC):
        ACM[:, i] = np.matrix.flatten(AdC[i*dimSUT[0]:dimSUT[0]*(1+i), 0:dimSUT[1]], order="F")
        ACMr[i] = AdC[i*dimSUT[0], dimSUT[1]]

    VCa = abs(VC)
    cv = VCa.sum(axis=0)
    cv[cv > 0] = 1
    cTs = np.arange(0, dimSUT[1], 1)+1
    tc1 = cv * cTs
    tc2 = (tc1[tc1 > 0])
    tc2 = tc2.astype(int)-1

    SUTv = np.matrix.flatten(SUT, order="F")
    ACv = np.matrix.flatten(AC, order="F")

    BM = np.transpose(np.kron(HC[0], np.eye(dimSUT[0])))

    VRM = np.zeros((tnV, dimSUT[1]))

    for i in range(0, dimSUT[1]):
        VRM[dimSUT[0]*(i):dimSUT[0]*(i+1), i] = VC[0:dimSUT[0], i]

    VRM = VRM[:, tc2]

    RM = np.hstack((BM, VRM, ACM))

    SUTv_hat = (SUTv * (1-ACv)) ** 2

    Ve = np.diag(SUTv_hat)

    Vd = np.dot(np.dot(np.transpose(RM), Ve), RM)
    Vd2 = RemZ(Vd)
    Vd_1 = np.dot(np.dot(np.transpose(np.squeeze(Vd2[1])),
                         np.linalg.pinv(np.squeeze(Vd2[0]))),
                         np.transpose((np.squeeze(Vd2[2]))))

    d1 = np.dot(np.transpose(SUTv), BM)
    d2 = np.dot(np.transpose(SUTv), VRM)
    d3 = np.dot(np.transpose(SUTv), ACM) - np.squeeze(ACMr)

    D = np.concatenate((d1, d2, d3))

    SUTd = SUTv + np.dot(np.dot(np.dot(Ve, RM), Vd_1), -D)

    SUTb = np.transpose(SUTd.reshape(dimSUT[1], dimSUT[0]))

    return(SUTb)


@xw.func
def RemZ(SP):

    SP2 = abs(SP)
    dimSP = np.shape(SP2)
    tc = SP2.sum(axis=0)
    tc[tc > 0] = 1
    tr = SP2.sum(axis=1)
    tr[tr > 0] = 1
    cI = np.identity(dimSP[1])
    rI = np.identity(dimSP[0])
    cTs = np.arange(0, dimSP[1], 1)+1
    rTs = np.arange(0, dimSP[0], 1)+1
    tc1 = tc * cTs
    tc2 = (tc1[tc1 > 0])
    tc2 = tc2.astype(int)-1
    cI2 = cI[:, tc2]
    tr1 = tr * rTs
    tr2 = (tr1[tr1 > 0])
    tr2 = tr2.astype(int)-1
    rI2 = rI[tr2, :]
    rSP = np.dot(np.dot(rI2, SP), cI2)

    OUT = [[rSP], [rI2], [cI2]]

    return(OUT)
